package nl.amis.demo.dynacols.model.revenue;

import java.math.BigDecimal;

import java.sql.SQLException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import nl.amis.demo.dynacols.model.revenue.types.RevenueData;

import nl.amis.demo.dynacols.model.revenue.types.RevenueDataTable;

import nl.amis.demo.dynacols.model.revenue.types.RevenuePeriodData;

import oracle.jbo.JboException;
import oracle.jbo.server.DBTransaction;

import oracle.jdbc.OracleCallableStatement;

import oracle.jdbc.OracleTypes;

import oracle.sql.ARRAY;
import oracle.sql.STRUCT;

public class RevenueHandler {
    public RevenueHandler() {
    }
    
        public static  List<RevenueData> getRevenue(String deptId, String year, DBTransaction dbt) { 
            String _getRevenueStatement = 
                "begin ? := hr_revenue_pkg.get_revenue(?,?); end;";
            ARRAY _nativeTypeRevenue = null;
            ARRAY _nativeTypePeriodes = null;
            OracleCallableStatement st = null;
            RevenueDataTable _revenueDataTable = 
                new RevenueDataTable(); 

            List<RevenueData> _revenueDataList = 
                new ArrayList<RevenueData>();
            try {
                st = 
            (OracleCallableStatement)dbt.createCallableStatement(_getRevenueStatement ,
                                                                         0);
                // Register the first bind variable for the return value
                st.registerOutParameter(1, OracleTypes.ARRAY, "REVENUE_TAB");

                st.setString(2, deptId);            
                st.setString(3, year);


                st.executeUpdate();
                // 6. Return the value of the first bind variable
                _nativeTypeRevenue = ((OracleCallableStatement)st).getARRAY(1);
                // the ARRAY object can yield an array of objects
                Object[] _revenueArray = (Object[])_nativeTypeRevenue.getArray();
                // the objects returned in the ARRAY are in fact STRUCTs
                // these are created based on the MATRIX SQL TYPE definition
                // each of the properties in the MATRIX TYPE correspond to a
                // an attribute in the STRUCT



                for (int i = 0; i < _revenueArray.length; i++) {
                    STRUCT _revenueStruct = (STRUCT)_revenueArray[i];


                    RevenueData _revenueData = 
                        new RevenueData();

                    _revenueData.setDepartmentId(_revenueStruct.getAttributes()[0].toString());
                    _revenueData.setDepartmentName(_revenueStruct.getAttributes()[1].toString());
                    
                    _nativeTypePeriodes = (ARRAY)_revenueStruct.getAttributes()[2];

    
                    

                    Object[] _revenuePeriodDataArray = 
                        (Object[])_nativeTypePeriodes.getArray();

                    List<RevenuePeriodData> _revenuePeriodDataList= 
                        new ArrayList<RevenuePeriodData>();
                    int _perIndex = 0;
                    Double _nullValue= new Double(0);
                    for (int j = 0; j < _revenuePeriodDataArray.length; j++) {
                        _perIndex++;
                        STRUCT periodeDataStruct = (STRUCT)_revenuePeriodDataArray[j];

                        RevenuePeriodData _periodData = 
                            new RevenuePeriodData();


                            
                        _periodData.setPeriodName(periodeDataStruct.getAttributes()[0].toString());
                        if (periodeDataStruct.getAttributes()[1]!=null){
                        _periodData.setRevenue(((BigDecimal)periodeDataStruct.getAttributes()[1]).doubleValue());
                        }
                        else{                       
                            _periodData.setRevenue(_nullValue);
                        }

                        _revenuePeriodDataList.add(_periodData);
                    }
                    _revenueData.setPeriodData(_revenuePeriodDataList);

                    _revenueDataList.add(_revenueData);

                    _revenueDataTable.setRevenueData(_revenueDataList);
                } // for
            } catch (SQLException e) {
                throw new JboException(e);

            } catch (RuntimeException e) {
                throw new JboException(e);
            }

            finally {
                if (st != null) {
                    try {
                        st.close();
                    } catch (SQLException e) {
                        throw new JboException(e);
                    }
                }
            }
            
            Collections.sort(_revenueDataList);
            
            return _revenueDataList;

        }
        
        public static String saveRevenue(List<RevenueData> newRevenue, DBTransaction dbt) { 
           
return "result";

        }

}
